
import typer
from mltk import cli



@cli.root_cli.command('view_audio')
def view_audio_command(
    model: str = typer.Argument(None, 
        help='''\b
One of the following:
- MLTK model name 
- Path to model specification file (.py)
- Path to model archive file (.mltk.zip)''',
        metavar='<model>'
    ),
    verbose: bool = typer.Option(False, '--verbose', '-v', 
        help='Enable verbose console logs'
    ),
):
    """View the spectrograms generated by the AudioFeatureGenerator in real-time

    NOTE: This command is experimental. Use at your own risk!
    
    This starts a wxPython GUI that allows for viewing the spectrograms generated
    by the AudioFeatureGenerator in real-time. 
    This is useful for determining which AudioFeatureGenerator settings and data
    augmentation parameters to use with the ParallelAudioDataGenerator.
    """
    # Import all required packages here instead of at top
    # to help improve the CLI's responsiveness
    from mltk.utils.audio_visualizer import run_audio_visualizer_app

    logger = cli.get_logger(verbose=verbose)
    run_audio_visualizer_app(model, logger=logger)